@@ -4,6 +4,7 @@ import logging
 import mimetypes
 import os
 import re
+import threading
 import weakref
 
 import httpx
@@ -33,6 +34,19 @@ HTTPX_RETRYABLE_HTTP_STATUS_CODES = (500, 502, 503, 504)
 _logger = logging.getLogger(__name__)
 
 
+def get_running_loop():
+    """Get the currently running event loop."""
+    # this was removed from fsspec in https://github.com/fsspec/filesystem_spec/pull/1134
+    if hasattr(asyncio, "get_running_loop"):
+        return asyncio.get_running_loop()
+    else:
+        loop = asyncio._get_running_loop()
+        if loop is None:
+            raise RuntimeError("no running event loop")
+        else:
+            return loop
+
+
 def parse_range_header(range_header):
     # Regular expression to match a range header like 'bytes=0-499'
     range_pattern = r"bytes=(\d+)?-(\d+)?"
@@ -126,13 +140,54 @@ class AbstractMSGraphFS(AsyncFileSystem):
             asynchronous=asynchronous, loop=loop or get_loop(), **super_kwargs
         )
 
-        self.client: AsyncOAuth2Client = AsyncOAuth2Client(
-            **oauth2_client_params,
+        # Store initialization parameters for lazy initialization
+        self._oauth2_client_params = oauth2_client_params
+        self._client = None
+        self._client_lock = threading.Lock() if not asynchronous else None
+        self._client_pid = None  # Track which process created the client
+        self.use_recycle_bin = kwargs.get("use_recycle_bin", False)
+
+    @property
+    def client(self) -> AsyncOAuth2Client:
+        """Lazy-initialized, fork-safe OAuth2 client."""
+        current_pid = os.getpid()
+
+        # Check if we need to initialize or reinitialize after fork
+        if self._client is None or self._client_pid != current_pid:
+            if self.asynchronous:
+                # For async mode, we can't use locks, but async is typically single-threaded
+                self._init_client()
+                self._client_pid = current_pid
+            else:
+                # Thread-safe lazy initialization for sync mode
+                with self._client_lock:
+                    # Double-check after acquiring lock
+                    if self._client is None or self._client_pid != current_pid:
+                        self._init_client()
+                        self._client_pid = current_pid
+
+        return self._client
+
+    def _init_client(self):
+        """Initialize the OAuth2 client."""
+        # Close existing client if it exists
+        if self._client is not None:
+            try:
+                # Try to close the old client gracefully
+                self.close_http_session(self._client, getattr(self, "loop", None))
+            except Exception:
+                # Ignore errors during cleanup
+                pass
+
+        # Create new client
+        self._client = AsyncOAuth2Client(
+            **self._oauth2_client_params,
             follow_redirects=True,
         )
+
+        # Register cleanup for non-async mode
         if not self.asynchronous:
-            weakref.finalize(self, self.close_http_session, self.client, self.loop)
-        self.use_recycle_bin = kwargs.get("use_recycle_bin", False)
+            weakref.finalize(self, self.close_http_session, self._client, self.loop)
 
     def __del__(self):
         """Destructor to ensure HTTP client is properly closed."""
@@ -143,25 +198,26 @@ class AbstractMSGraphFS(AsyncFileSystem):
             # Ignore all cleanup errors in destructor
             pass
 
+    def _get_loop(self):
+        """Get the current event loop, following adlfs pattern."""
+        try:
+            # Need to confirm there is an event loop running in
+            # the thread. If not, create the fsspec loop
+            # and set it. This is to handle issues with
+            # Async Credentials from the Azure SDK
+            loop = get_running_loop()
+        except RuntimeError:
+            from fsspec.asyn import get_loop
+
+            loop = get_loop()
+            asyncio.set_event_loop(loop)
+
+        return loop
+
     @property
     def loop(self):
-        """Fork-safe loop property that reinitializes the loop after fork detection."""
-        if self._pid != os.getpid():
-            # Fork detected - reinitialize the loop for the new process
-            self._pid = os.getpid()
-            if not self.asynchronous:
-                from fsspec.asyn import get_loop
-
-                self._loop = get_loop()
-                # Also need to set the event loop for the new process
-                import asyncio
-
-                try:
-                    asyncio.set_event_loop(self._loop)
-                except RuntimeError:
-                    # Event loop might already be set
-                    pass
-        return self._loop
+        """Get the event loop for this filesystem."""
+        return self._get_loop()
 
     @staticmethod
     def close_http_session(
@@ -262,8 +318,122 @@ class AbstractMSGraphFS(AsyncFileSystem):
         if "fields" in drive_item_info:
             data["fields"] = drive_item_info["fields"]
 
+        # Add permissions if they were expanded/included in the response
+        if "permissions" in drive_item_info:
+            data["permissions"] = self._format_permissions(
+                drive_item_info["permissions"]
+            )
+
         return data
 
+    def _format_permissions(self, permissions: list) -> dict:
+        """Format permissions from Microsoft Graph API into a more readable structure.
+
+        Args:
+            permissions: List of permission objects from Graph API
+
+        Returns:
+            dict: Formatted permissions with users, groups, and access levels
+        """
+        if not permissions:
+            return {
+                "users": [],
+                "groups": [],
+                "links": [],
+                "summary": {"total_permissions": 0},
+            }
+
+        users = []
+        groups = []
+        links = []
+
+        for perm in permissions:
+            perm_info = {
+                "id": perm.get("id"),
+                "roles": perm.get("roles", []),
+                "expires": perm.get("expirationDateTime"),
+                "has_password": perm.get("hasPassword", False),
+            }
+
+            # Handle different grantee types
+            granted_to = perm.get("grantedTo")
+            granted_to_identities = perm.get("grantedToIdentities", [])
+
+            if granted_to:
+                # Direct user/group permission
+                if granted_to.get("user"):
+                    user_info = granted_to["user"]
+                    users.append(
+                        {
+                            **perm_info,
+                            "type": "user",
+                            "email": user_info.get("email"),
+                            "display_name": user_info.get("displayName"),
+                            "id": user_info.get("id"),
+                        }
+                    )
+                elif granted_to.get("group"):
+                    group_info = granted_to["group"]
+                    groups.append(
+                        {
+                            **perm_info,
+                            "type": "group",
+                            "email": group_info.get("email"),
+                            "display_name": group_info.get("displayName"),
+                            "id": group_info.get("id"),
+                        }
+                    )
+
+            # Handle multiple identities (e.g., for sharing links)
+            for identity in granted_to_identities:
+                if identity.get("user"):
+                    user_info = identity["user"]
+                    users.append(
+                        {
+                            **perm_info,
+                            "type": "user",
+                            "email": user_info.get("email"),
+                            "display_name": user_info.get("displayName"),
+                            "id": user_info.get("id"),
+                        }
+                    )
+                elif identity.get("group"):
+                    group_info = identity["group"]
+                    groups.append(
+                        {
+                            **perm_info,
+                            "type": "group",
+                            "email": group_info.get("email"),
+                            "display_name": group_info.get("displayName"),
+                            "id": group_info.get("id"),
+                        }
+                    )
+
+            # Handle sharing links
+            link = perm.get("link")
+            if link:
+                links.append(
+                    {
+                        **perm_info,
+                        "type": "link",
+                        "link_type": link.get("type"),  # e.g., "view", "edit", "embed"
+                        "scope": link.get("scope"),  # e.g., "anonymous", "organization"
+                        "web_url": link.get("webUrl"),
+                    }
+                )
+
+        return {
+            "users": users,
+            "groups": groups,
+            "links": links,
+            "summary": {
+                "total_permissions": len(permissions),
+                "user_count": len(users),
+                "group_count": len(groups),
+                "link_count": len(links),
+            },
+        }
+
     async def _get_item_id(self, path: str, throw_on_missing=False) -> str | None:
         """Get the item ID of a file or directory.
 
@@ -1004,6 +1174,48 @@ class AbstractMSGraphFS(AsyncFileSystem):
 
     set_properties = sync_wrapper(_set_properties)
 
+    async def _get_permissions(self, path: str, item_id: str | None = None) -> dict:
+        """Get detailed permissions for a file or directory.
+
+        This method fetches the permissions from the Microsoft Graph API and formats them
+        into a more readable structure showing users, groups, and sharing links with their
+        respective access levels.
+
+        Parameters
+        ----------
+        path : str
+            Path of the file or directory to get permissions for
+        item_id: str
+            If given, the item_id will be used instead of the path to get
+            the permissions for the file or directory.
+
+        Returns
+        -------
+        dict
+            Formatted permissions with users, groups, links, and summary information
+
+        Examples
+        --------
+        >>> permissions = fs.get_permissions("/documents/important.docx")
+        >>> print(f"Total permissions: {permissions['summary']['total_permissions']}")
+        >>> for user in permissions['users']:
+        ...     print(f"User: {user['display_name']} - Roles: {user['roles']}")
+        """
+        url = await self._path_to_url_async(path, item_id=item_id, action="permissions")
+        response = await self._msgraph_get(url)
+        result = response.json()
+        permissions = result.get("value", [])
+
+        # Handle pagination
+        while "@odata.nextLink" in result:
+            response = await self._msgraph_get(result["@odata.nextLink"])
+            result = response.json()
+            permissions.extend(result.get("value", []))
+
+        return self._format_permissions(permissions)
+
+    get_permissions = sync_wrapper(_get_permissions)
+
 
 class MSGDriveFS(AbstractMSGraphFS):
     """A filesystem that represents a SharePoint site drive as a filesystem.
